@@ -42,14 +42,8 @@
 #define d(x)
 
 /*Prototypes*/
-static gboolean ews_info_set_flags (CamelMessageInfo *info, guint32 flags, guint32 set);
-
-static gboolean summary_header_from_db (CamelFolderSummary *s, CamelFIRecord *mir);
-static CamelFIRecord * summary_header_to_db (CamelFolderSummary *s, GError **error);
-static CamelMIRecord * message_info_to_db (CamelFolderSummary *s, CamelMessageInfo *info);
-static CamelMessageInfo * message_info_from_db (CamelFolderSummary *s, CamelMIRecord *mir);
-static gboolean content_info_to_db (CamelFolderSummary *s, CamelMessageContentInfo *info, CamelMIRecord *mir);
-static CamelMessageContentInfo * content_info_from_db (CamelFolderSummary *s, CamelMIRecord *mir);
+static gboolean summary_header_load (CamelFolderSummary *s, CamelFIRecord *mir);
+static CamelFIRecord * summary_header_save (CamelFolderSummary *s, GError **error);
 
 /*End of Prototypes*/
 
@@ -61,34 +55,6 @@ struct _CamelEwsSummaryPrivate {
 
 G_DEFINE_TYPE (CamelEwsSummary, camel_ews_summary, CAMEL_TYPE_FOLDER_SUMMARY)
 
-static CamelMessageInfo *
-ews_message_info_clone (CamelFolderSummary *s,
-                        const CamelMessageInfo *mi)
-{
-	CamelEwsMessageInfo *to;
-	const CamelEwsMessageInfo *from = (const CamelEwsMessageInfo *) mi;
-
-	to = (CamelEwsMessageInfo *) CAMEL_FOLDER_SUMMARY_CLASS (camel_ews_summary_parent_class)->message_info_clone (s, mi);
-	to->server_flags = from->server_flags;
-	to->item_type = from->item_type;
-	to->change_key = g_strdup (from->change_key);
-
-	/* FIXME: parent clone should do this */
-	to->info.content = camel_folder_summary_content_info_new (s);
-
-	return (CamelMessageInfo *) to;
-}
-
-static void
-ews_message_info_free (CamelFolderSummary *s,
-                       CamelMessageInfo *mi)
-{
-	CamelEwsMessageInfo *emi = (gpointer) mi;
-
-	g_free (emi->change_key);
-	CAMEL_FOLDER_SUMMARY_CLASS (camel_ews_summary_parent_class)->message_info_free (s, mi);
-}
-
 static void
 ews_summary_finalize (GObject *object)
 {
@@ -113,17 +79,9 @@ camel_ews_summary_class_init (CamelEwsSummaryClass *class)
 	object_class->finalize = ews_summary_finalize;
 
 	folder_summary_class = CAMEL_FOLDER_SUMMARY_CLASS (class);
-	folder_summary_class->message_info_size = sizeof (CamelEwsMessageInfo);
-	folder_summary_class->content_info_size = sizeof (CamelEwsMessageContentInfo);
-	folder_summary_class->message_info_clone = ews_message_info_clone;
-	folder_summary_class->message_info_free = ews_message_info_free;
-	folder_summary_class->info_set_flags = ews_info_set_flags;
-	folder_summary_class->summary_header_to_db = summary_header_to_db;
-	folder_summary_class->summary_header_from_db = summary_header_from_db;
-	folder_summary_class->message_info_to_db = message_info_to_db;
-	folder_summary_class->message_info_from_db = message_info_from_db;
-	folder_summary_class->content_info_to_db = content_info_to_db;
-	folder_summary_class->content_info_from_db = content_info_from_db;
+	folder_summary_class->message_info_type = CAMEL_TYPE_EWS_MESSAGE_INFO;
+	folder_summary_class->summary_header_save = summary_header_save;
+	folder_summary_class->summary_header_load = summary_header_load;
 }
 
 static void
@@ -148,21 +106,20 @@ camel_ews_summary_new (struct _CamelFolder *folder)
 	CamelFolderSummary *summary;
 
 	summary = g_object_new (CAMEL_TYPE_EWS_SUMMARY, "folder", folder, NULL);
-	camel_folder_summary_set_build_content (summary, TRUE);
 
-	camel_folder_summary_load_from_db (summary, NULL);
+	camel_folder_summary_load (summary, NULL);
 
 	return summary;
 }
 
 static gboolean
-summary_header_from_db (CamelFolderSummary *s,
-                        CamelFIRecord *mir)
+summary_header_load (CamelFolderSummary *s,
+		     CamelFIRecord *mir)
 {
 	CamelEwsSummary *ews_summary = CAMEL_EWS_SUMMARY (s);
 	gchar *part;
 
-	if (!CAMEL_FOLDER_SUMMARY_CLASS (camel_ews_summary_parent_class)->summary_header_from_db (s, mir))
+	if (!CAMEL_FOLDER_SUMMARY_CLASS (camel_ews_summary_parent_class)->summary_header_load (s, mir))
 		return FALSE;
 
 	part = mir->bdata;
@@ -178,14 +135,14 @@ summary_header_from_db (CamelFolderSummary *s,
 }
 
 static CamelFIRecord *
-summary_header_to_db (CamelFolderSummary *s,
-                      GError **error)
+summary_header_save (CamelFolderSummary *s,
+		     GError **error)
 {
 	CamelEwsSummary *ews_summary = CAMEL_EWS_SUMMARY (s);
 	struct _CamelFIRecord *fir;
 	gchar *sync_state;
 
-	fir = CAMEL_FOLDER_SUMMARY_CLASS (camel_ews_summary_parent_class)->summary_header_to_db (s, error);
+	fir = CAMEL_FOLDER_SUMMARY_CLASS (camel_ews_summary_parent_class)->summary_header_save (s, error);
 	if (!fir)
 		return NULL;
 
@@ -199,95 +156,6 @@ summary_header_to_db (CamelFolderSummary *s,
 
 }
 
-static CamelMessageInfo *
-message_info_from_db (CamelFolderSummary *s,
-                      CamelMIRecord *mir)
-{
-	CamelMessageInfo *info;
-	CamelEwsMessageInfo *iinfo;
-
-	info = CAMEL_FOLDER_SUMMARY_CLASS (camel_ews_summary_parent_class)->message_info_from_db (s, mir);
-	if (info) {
-		gchar *part = mir->bdata;
-		gchar **values;
-
-		iinfo = (CamelEwsMessageInfo *) info;
-		if (part) {
-			values = g_strsplit (part, " ", -1);
-
-			if (values && values[0] && values[1] && values[2]) {
-				iinfo->server_flags = g_ascii_strtoll (values[0], NULL, 10);
-				iinfo->item_type = g_ascii_strtoll (values[1], NULL, 10);
-				iinfo->change_key = g_strdup (values[2]);
-			}
-
-			g_strfreev (values);
-		}
-	}
-
-	return info;
-}
-
-static CamelMIRecord *
-message_info_to_db (CamelFolderSummary *s,
-                    CamelMessageInfo *info)
-{
-	CamelEwsMessageInfo *iinfo = (CamelEwsMessageInfo *) info;
-	struct _CamelMIRecord *mir;
-
-	mir = CAMEL_FOLDER_SUMMARY_CLASS (camel_ews_summary_parent_class)->message_info_to_db (s, info);
-	if (mir)
-		mir->bdata = g_strdup_printf ("%u %d %s", iinfo->server_flags, iinfo->item_type, iinfo->change_key);
-
-	return mir;
-}
-
-static CamelMessageContentInfo *
-content_info_from_db (CamelFolderSummary *s,
-                      CamelMIRecord *mir)
-{
-	gchar *part = mir->cinfo;
-	guint32 type = 0;
-
-	if (part) {
-		if (*part == ' ')
-			part++;
-		if (part) {
-			EXTRACT_FIRST_DIGIT (type);
-		}
-	}
-	mir->cinfo = part;
-	if (type)
-		return CAMEL_FOLDER_SUMMARY_CLASS (camel_ews_summary_parent_class)->content_info_from_db (s, mir);
-	else
-		return camel_folder_summary_content_info_new (s);
-}
-
-static gboolean
-content_info_to_db (CamelFolderSummary *s,
-                    CamelMessageContentInfo *info,
-                    CamelMIRecord *mir)
-{
-
-	if (info->type) {
-		g_free (mir->cinfo);
-		mir->cinfo = g_strdup ("1");
-		return CAMEL_FOLDER_SUMMARY_CLASS (camel_ews_summary_parent_class)->content_info_to_db (s, info, mir);
-	} else {
-		g_free (mir->cinfo);
-		mir->cinfo = g_strdup ("0");
-		return TRUE;
-	}
-}
-
-static gboolean
-ews_info_set_flags (CamelMessageInfo *info,
-                    guint32 flags,
-                    guint32 set)
-{
-	return CAMEL_FOLDER_SUMMARY_CLASS (camel_ews_summary_parent_class)->info_set_flags (info, flags, set);
-}
-
 gboolean
 camel_ews_summary_add_message (CamelFolderSummary *summary,
                                const gchar *uid,
@@ -295,67 +163,55 @@ camel_ews_summary_add_message (CamelFolderSummary *summary,
 			       CamelMessageInfo *info,
                                CamelMimeMessage *message)
 {
-	CamelEwsMessageInfo *mi;
-	const CamelFlag *flag;
-	const CamelTag *tag;
+	CamelMessageInfo *mi;
 
 	g_return_val_if_fail (uid != NULL, FALSE);
 	g_return_val_if_fail (info != NULL, FALSE);
 	g_return_val_if_fail (message != NULL, FALSE);
 
 	/* Create summary entry */
-	mi = (CamelEwsMessageInfo *) camel_folder_summary_info_new_from_message (summary, message, NULL);
+	mi = camel_folder_summary_info_new_from_message (summary, message);
 	g_return_val_if_fail (mi != NULL, FALSE);
 
-	/* Set the change_key */
-	mi->change_key = g_strdup (change_key);
+	camel_message_info_set_abort_notifications (mi, TRUE);
 
-	/* Copy flags 'n' tags */
-	mi->info.flags = camel_message_info_get_flags (info);
+	camel_ews_message_info_set_change_key (CAMEL_EWS_MESSAGE_INFO (mi), change_key);
+	camel_message_info_set_flags (mi, ~0, camel_message_info_get_flags (info));
+	camel_message_info_take_user_flags (mi, camel_message_info_dup_user_flags (info));
+	camel_message_info_take_user_tags (mi, camel_message_info_dup_user_tags (info));
+	camel_message_info_set_size (mi, camel_message_info_get_size (info));
+	camel_message_info_set_uid (mi, uid);
 
-	flag = camel_message_info_get_user_flags (info);
-	while (flag) {
-		camel_message_info_set_user_flag ((CamelMessageInfo *) mi, flag->name, TRUE);
-		flag = flag->next;
-	}
+	camel_message_info_set_abort_notifications (mi, FALSE);
 
-	tag = camel_message_info_get_user_tags (info);
-	while (tag) {
-		/* coverity[unchecked_value] */
-		camel_message_info_set_user_tag ((CamelMessageInfo *) mi, tag->name, tag->value);
-		tag = tag->next;
-	}
-
-	mi->info.size = camel_message_info_get_size (info);
-	mi->info.uid = camel_pstring_strdup (uid);
-
-	camel_folder_summary_add (summary, (CamelMessageInfo *) mi);
+	camel_folder_summary_add (summary, mi, FALSE);
 	camel_folder_summary_touch (summary);
-	camel_folder_summary_save_to_db (summary, NULL);
+	camel_folder_summary_save (summary, NULL);
+
+	g_object_unref (mi);
 
 	return TRUE;
 }
 
 static gboolean
 ews_update_user_flags (CamelMessageInfo *info,
-                       CamelFlag *server_user_flags)
+                       const CamelNamedFlags *server_user_flags)
 {
 	gboolean changed = FALSE;
-	CamelMessageInfoBase *binfo = (CamelMessageInfoBase *) info;
 	gboolean set_cal = FALSE, set_note = FALSE;
 
-	if (camel_flag_get (&binfo->user_flags, "$has_cal"))
+	if (camel_message_info_get_user_flag (info, "$has_cal"))
 		set_cal = TRUE;
-	if (camel_flag_get (&binfo->user_flags, "$has_note"))
+	if (camel_message_info_get_user_flag (info, "$has_note"))
 		set_note = TRUE;
 
-	changed = camel_flag_list_copy (&binfo->user_flags, &server_user_flags);
+	changed = camel_message_info_take_user_flags (info, camel_named_flags_copy (server_user_flags));
 
 	/* reset the flags as they were set in messageinfo before */
 	if (set_cal)
-		camel_flag_set (&binfo->user_flags, "$has_cal", TRUE);
+		camel_message_info_set_user_flag (info, "$has_cal", TRUE);
 	if (set_note)
-		camel_flag_set (&binfo->user_flags, "$has_note", TRUE);
+		camel_message_info_set_user_flag (info, "$has_note", TRUE);
 
 	return changed;
 }
@@ -364,21 +220,24 @@ gboolean
 camel_ews_update_message_info_flags (CamelFolderSummary *summary,
                                      CamelMessageInfo *info,
                                      guint32 server_flags,
-                                     CamelFlag *server_user_flags)
+                                     const CamelNamedFlags *server_user_flags)
 {
-	CamelEwsMessageInfo *einfo = (CamelEwsMessageInfo *) info;
+	CamelEwsMessageInfo *emi;
 	gboolean changed = FALSE;
 
-	if (server_flags != einfo->server_flags) {
+	g_return_val_if_fail (CAMEL_IS_EWS_SUMMARY (summary), FALSE);
+	g_return_val_if_fail (CAMEL_IS_EWS_MESSAGE_INFO (info), FALSE);
+
+	emi = CAMEL_EWS_MESSAGE_INFO (info);
+
+	if (server_flags != camel_ews_message_info_get_server_flags (emi)) {
 		guint32 server_set, server_cleared;
 
-		server_set = server_flags & ~einfo->server_flags;
-		server_cleared = einfo->server_flags & ~server_flags;
+		server_set = server_flags & ~camel_ews_message_info_get_server_flags (emi);
+		server_cleared = camel_ews_message_info_get_server_flags (emi) & ~server_flags;
 
-		camel_message_info_set_flags (info, server_set | server_cleared, (einfo->info.flags | server_set) & ~server_cleared);
-		einfo->server_flags = server_flags;
-		if (info->summary)
-			camel_folder_summary_touch (info->summary);
+		camel_message_info_set_flags (info, server_set | server_cleared, (camel_message_info_get_flags (info) | server_set) & ~server_cleared);
+		camel_ews_message_info_set_server_flags (emi, server_flags);
 		changed = TRUE;
 	}
 
